#!/usr/bin/perl
# mysql-rrd.pl
# v1.0 - Nicolas AGIUS - 04/2007
#
# Usage: mysql-rrd.pl [init]
#		init : Create new empty RRD file
#
###################################################################################
## MySQL performances sensor for RRDtools
#
# Copyright (C) 2007 Nicolas AGIUS <nicolas_agius@yahoo.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
###################################################################################

use DBI;
use RRDs;
use strict;

# Configuration variables
###################################################################################

my $mysql_host = "localhost";				# MySQL server to monitor
my $mysql_user = "root";					# MySQL user with "SHOW STATUS" privilege
my $mysql_pass = "";						# MySQL user's password
my $rrd_file   = "/var/lib/mysql.rrd";		# Round Robin Database file


## CODE START HERE
###################################################################################

if($#ARGV>=0 && $ARGV[0] ne "init")
{
	print "Usage : $0 [init]
    (none) : Update RRD with MySQL status.
    init   : Create new empty RRD file.
";
	exit 3;
}


# Create new RRD file
###################################################################################

if($ARGV[0] eq "init")
{
	# Override security
	if(-e $rrd_file)
	{
		print "WARNING : $rrd_file already exist ! Override ? [y/N]";
		my $ask = <STDIN>;
		chomp($ask);
		if($ask ne "Y" and $ask ne "y")
		{
			print "Aborded by user.\n";
			exit;
		}
	}

	print "Creating new RRD file...\n";
	RRDs::create("$rrd_file",
		"DS:select:COUNTER:600:0:U",		# Com_select
		"DS:insert:COUNTER:600:0:U",		# Com_insert
		"DS:update:COUNTER:600:0:U",		# Com_update
		"DS:delete:COUNTER:600:0:U",		# Com_delete
		"DS:cache:COUNTER:600:0:U",			# Qcache_hits		
		"DS:total:COUNTER:600:0:U",			# Questions
		"DS:connect:COUNTER:600:0:U",		# Connections
		"DS:inbound:COUNTER:600:0:U",		# Bytes_received
		"DS:outbound:COUNTER:600:0:U",		# Bytes_sent
		"RRA:AVERAGE:0.5:1:600",			# 1 sample "averaged" stays 1 period of 5 minutes
		"RRA:AVERAGE:0.5:6:700",			# 6 samples averaged become one average on 30 minutes
		"RRA:AVERAGE:0.5:24:775",			# 24 samples averaged become one average on 2 hours
		"RRA:AVERAGE:0.5:288:797",			# 288 samples averaged become one average on 1 day
		"RRA:MAX:0.5:1:600",				# 600 samples of 5 minutes  (2 days and 2 hours)
		"RRA:MAX:0.5:6:700",				# 700 samples of 30 minutes (2 days and 2 hours, plus 12.5 days)
		"RRA:MAX:0.5:24:775",				# 775 samples of 2 hours    (above + 50 days)
		"RRA:MAX:0.5:288:797"				# 797 samples of 1 day      (above + 732 days, rounded up to 797)
	);
	
	check_rrd_error();
	exit;	
}


# Updating database
###################################################################################

# For details about this mysql status command, 
# See 
#    http://bugs.mysql.com/bug.php?id=19093 
# and
#    http://dev.mysql.com/doc/refman/5.0/en/show-status.html
#
my $sql="SHOW /*!50002 GLOBAL */ STATUS";

# Connect to database
my $dbh = DBI->connect("DBI:mysql::$mysql_host", $mysql_user, $mysql_pass) 
	or die "ERR: Couldn't connect to database: " . DBI->errstr;

# Fetch datas in a hash table (row 1 is the key)
my $hr_status = $dbh->selectall_hashref($sql,1) 
	or die "ERR: Couldn't execute statement: " . $dbh->errstr;

# Feed RRD
RRDs::update("$rrd_file","N:".
	get("Com_select").":".
	get("Com_insert").":".
	get("Com_update").":".
	get("Com_delete").":".
	get("Qcache_hits").":".
	get("Questions").":".
	get("Connections").":".
	get("Bytes_received").":".
	get("Bytes_sent")
);
check_rrd_error();

# Clean exit
$dbh->disconnect();
exit 0;


# Subs
###################################################################################

# RRD error handler
sub check_rrd_error
{
	my $ERR=RRDs::error();
	die "ERR: $ERR\n" if $ERR;
}
		
# Data checker: UNKNOWN if not defined
sub get
{
	my $data = $hr_status->{$_[0]}->{'Value'};
	return defined($data) ? $data : "U";
}

# EOF
