#!/bin/bash
# mysql-rrd.sh
# v1.0 - Nicolas AGIUS - 05/2007
#
# Usage: mysql-rrd.sh [init]
#       init : Create new empty RRD file
#
###################################################################################
## MySQL performances sensor for RRDtools
#
# Copyright (C) 2007 Nicolas AGIUS <nicolas_agius@yahoo.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
###################################################################################

# Configuration variables
###################################################################################

MYSQL_HOST="localhost";               # MySQL server to monitor
MYSQL_USER="root";                    # MySQL user with "SHOW STATUS" privilege
MYSQL_PASS="";						  # MySQL user's password
RRD_FILE="/var/lib/mysql.rrd";    	  # Round Robin Database file

## CODE START HERE
###################################################################################

if [ $# -gt 0 -a "$1" != "init" ]; then
    echo "Usage : $0 [init]
    (none) : Update RRD with MySQL status.
    init   : Create new empty RRD file.
"
    exit 3
fi


# Create new RRD file
###################################################################################

if [ "$1" == "init" ]; then
	# Override security
	if [ -f $RRD_FILE ]; then
		read -p "WARNING : $RRD_FILE already exist ! Override ? [y/N]" J
		if [ "$J" != "Y" -a "$J" != "y" ]; then
			 echo "Aborded by user."
			 exit
		fi
	fi

	echo "Creating new RRD file..."
	rrdtool create $RRD_FILE   \
           DS:select:COUNTER:600:0:U    \
           DS:insert:COUNTER:600:0:U    \
           DS:update:COUNTER:600:0:U    \
           DS:delete:COUNTER:600:0:U   	\
           DS:cache:COUNTER:600:0:U   	\
           DS:total:COUNTER:600:0:U   	\
		   DS:connect:COUNTER:600:0:U  	\
           DS:inbound:COUNTER:600:0:U   \
           DS:outbound:COUNTER:600:0:U  \
           RRA:AVERAGE:0.5:1:600      	\
           RRA:AVERAGE:0.5:6:700      	\
           RRA:AVERAGE:0.5:24:775     	\
           RRA:AVERAGE:0.5:288:797    	\
           RRA:MAX:0.5:1:600          	\
           RRA:MAX:0.5:6:700          	\
           RRA:MAX:0.5:24:775         	\
           RRA:MAX:0.5:288:797			
	exit
fi


# Updating database
###################################################################################

# Initialisation with default
select="U"
insert="U"
update="U"
delete="U"
cache="U"
total="U"
connect="U"
inbound="U"
outbound="U"

# Connect to database
mysqladmin --user=$MYSQL_USER --password=$MYSQL_PASS extended-status | ( while read DUMMY KEY DUMMY VALUE DUMMY
	do
		case $KEY in
			"Com_select")
				select=$VALUE
				;;
			"Com_insert")
				insert=$VALUE
				;;
			"Com_update")
				update=$VALUE
				;;
			"Com_delete")
				delete=$VALUE
				;;
			"Qcache_hits")
				cache=$VALUE
				;;
			"Questions")
				total=$VALUE
				;;
			"Connections")
				connect=$VALUE
				;;
			"Bytes_received")
				inbound=$VALUE
				;;
			"Bytes_sent")
				outbound=$VALUE
				;;
		esac
	done

	# Feed RRD
	rrdtool update $RRD_FILE N:$select:$insert:$update:$delete:$cache:$total:$connect:$inbound:$outbound
)

exit

# EOF

