<?
# rrd.inc.php [MyRRD]
# v 1.2 - Nicolas AGIUS - 06/2007
# 
# Changelog: 01/2011: v1.2: Added zoom.
#
###################################################################################
## Generate HTML page and graphics from RRDtools
#
# Copyright (C) 2007-2011 Nicolas AGIUS <nicolas_agius@yahoo.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
###################################################################################

// Configuration

/**********************************************************************************

 Graphics'datas definitions have to be something like this :

$graphs = array
(
    "graph1" => array // First graph
    (
        "Label"   => "Title of the day-graph box in the index page",
        "Title"   => "Title of the HTML page",
        "Caption" => "Caption of the HTML page",
        "Graph"   => array
        (
            "VLabel"    => "Vertical label of the graph",
            "HLabel"    => "Title of the graph",
            "Prefix"    => "graph1",  	// Prefix for the image filename
			"LowerLimit" => 0,			// OPTIONAL : Set the lower limit of the y-axis
            "UpperLimit" => 100,		// OPTIONAL : Set the upper limit of the y-axis
            "Data"      => array		// RRD's Data definitions
            (
                array				// First line
                (
                    "DSName"  => "ds-name",		// RRD data source name
                    "RRDFile" => "data.rrd",	// RRD file
                    "Label"   => "ds-label",	// Label of the line
                    "Color"   => "#FF0000",		// Color of the line
                    "Type"    => "LINE1"		// RRD type of the line (may be AREA)
					"Stack"   => true,			// OPTIONAL : Stack current data with previous one
					"Compute" => "100,*",		// OPTIONAL : Compute new values to draw from current data source
                ),
                array ... next line definition
            )
        )
    ),
    "graph2" => array ... next graph definition
);

**********************************************************************************/

// Globals vars
//-------------


$debug = false;		//debugging option

// Times definitions	
$schedule = array
(
	"Time" =>  array
	(
		"day"	=> "1day3hours",
		"week"	=> "1week1day",
		"month"	=> "1month2day",
		"year"	=> "1year1month"
	),
	"Timestamps" =>  array
	(
		"day"	=> "-1 day -3 hours",
		"week"	=> "-1 week -1 day",
		"month"	=> "-1 month -2 days",
		"year"	=> "-1 year -1 month"
	),
	"Label" => array
	(
		"day"	=> "quotidien",
		"week"	=> "hebdomadaire (moyenne)",
		"month"	=> "mensuel (moyenne)",
		"year"	=> "annuel (moyenne et maximum)"
	)
);

// Colors definitions
$colors = array
(
	"Title" 		=> "#B7E3A0",
	"Cell"	 		=> "#FFEE88",
	"Slider"		=> "#EEEEEE",
	"Footer" 		=> "#FFFFCC",
	"Background" 	=> "#FFFFCC"
);

// Code start here
//----------------

/****************************************************************************
 ** Main function
 *
 *
 *	Create graphs and HTML page like MRTG's style from RRD databases
 *
 *	Parametres :
 *
 *	$graphs   (array) : Graphics definitons described above
 *	$title   (string) : Title of the main index page
 *	$caption (string) : Caption of the main index page
 *	
 * It use GET parameters 'action', 'target' and 'refresh'
 *
 ****************************************************************************/
function rrd_mainhtml($graphs,$title,$caption)
{
	// Inputs Check
	sanitize_inputs($graphs);

	// Delete all olds graphs
	if(isset($_GET['refresh']) && $_GET['refresh']=="true") 
	{
		if(isset($_GET['target']))
		{
			// Delete old graphs ('cause RRD is in lazy mode)
			delete_graph($graphs[$_GET['target']]['Graph']);
		}
		else
		{
			foreach($graphs as $name => $info_graph)
			{
				delete_graph($info_graph['Graph']);
			}
		}
	}

	// Switch between specified mode
	if(isset($_GET['action']) && $_GET['action']=="details") // Full page for specified target
	{
		create_page($graphs[$_GET['target']]);
	}
	elseif(isset($_GET['action']) && $_GET['action']=="zoom")
	{
		create_zoom($graphs[$_GET['target']], $_GET['begin'], $_GET['end']);
	}
	else	
	{
		create_index($graphs,$title,$caption); // Index page with all graphs
	}
}


// Subs definitions
//---------------------------------------------------------------------------------


// Security checks
function sanitize_inputs($graphs)
{
	if(isset($_GET['target']))
    { 
		// Sanitize input
		ereg("^[a-zA-Z0-9_\-]+$", $_GET['target']) or die("Error: forbidden input.");

		// Validate input 	
		isset($graphs[$_GET['target']]) or die("Error: Invalid input.");
	}

	// Check action zoom 
	if(isset($_GET['action']) && $_GET['action']=="zoom" && !(isset($_GET['begin']) && isset($_GET['end'])))
		unset($_GET['action']);

	if(isset($_GET['begin']) && isset($_GET['end']))
	{
		// Translate dates to timestamps
		ereg("^[0-9]+$", $_GET['begin']) or $_GET['begin']=strtotime($_GET['begin']);
		ereg("^[0-9]+$", $_GET['end']) or $_GET['end']=strtotime($_GET['end']);

		// Check timestamps
		if($_GET['begin'] >= $_GET['end'])
			die("Error: bad timestamps.");		
	}
}


// Create an RRD graph for the specified data and period
// Return the filename of the image
function create_graph($gd,$period,$small=false,$begin=NULL,$end=NULL)
{
	global $schedule;
	global $debug;

	if($debug) print_r($gd);

	$filename = $gd['Prefix']."-".$period.".png";

	$gs = "
		$filename
		--vertical-label \"$gd[VLabel]\"
		--title \"".(strlen($gd['HLabel'])>0?$gd['HLabel']:"")."\"
		";

	if(is_null($begin) || is_null($end))
		$gs .= " --start -".$schedule['Time'][$period]." "; // Pre-defined times
	else
		$gs .= " --start $begin --end $end ";

	// Select graph' size
	$gs .= $small ? "--width 460 --height 100 " : "--width 800 --height 200 ";

	if(isset($gd['LowerLimit'])) $gs .= "--lower-limit $gd[LowerLimit] ";
	if(isset($gd['UpperLimit'])) $gs .= "--upper-limit $gd[UpperLimit] ";
 
	foreach($gd['Data'] as $d)
	{
		// Unique vname
		$n = $d['DSName'].dechex(rand(0,65535));

		// Define data source
		$gs .= "DEF:$n=$d[RRDFile]:$d[DSName]:AVERAGE ";

		// Switch to computed value if any
		if(isset($d['Compute']))
		{
			$gs .= "CDEF:$n-c=$n,$d[Compute] ";
			$n="$n-c";
		}

		// Check if we have to stack graph with previous line
		$s = (isset($d['Stack']) && $d['Stack'])?":STACK":"";

		// Draw values and legend
		$gs .= "
			$d[Type]:$n$d[Color]:\"$d[Label]\"$s
			GPRINT:$n:MAX:\"\\tMax\: %6.2lf%s\"
			GPRINT:$n:MIN:\"Min\: %6.2lf%s\"
			GPRINT:$n:AVERAGE:\"Moy\: %6.2lf%s\"
			GPRINT:$n:LAST:\"Cur\: %6.2lf%s\\n\"
		";
	}

	//cleanup params
	$gs=strtr($gs,"\n"," ");	
	
	//check safe_mode status (if in safe_mode, need to put a symlink to rrdtool in safe_mode_exec_dir)
	$rrd_cmd=ini_get('safe_mode')?"rrdtool":"/usr/bin/rrdtool";
	
	//execute RRDtool
	if($debug)
	{
		print_r($gs);
		syscall("$rrd_cmd graph $gs");
	}
	else
	{
		exec("$rrd_cmd graph $gs") or die("RRD error");
	}

	return $filename;
}


// system() like with stderr reporting
function syscall($command)
{
	$retval = "\nDEBUG output :\n";

	$descriptorspec = array(
		0 => array('pipe', 'r'),
		1 => array('pipe', 'w'),
		2 => array('pipe', 'w')
	);

	$resource = proc_open($command, $descriptorspec, $pipes, null, $_ENV);
	if (is_resource($resource))
	{
		$stdin = $pipes[0];
		$stdout = $pipes[1];
		$stderr = $pipes[2];

		while (! feof($stdout))
		{
			$retval .= fgets($stdout);
		}

		while (! feof($stderr))
		{
			$retval .= fgets($stderr);
		}

		fclose($stdin);
		fclose($stdout);
		fclose($stderr);

		$exit_code = proc_close($resource);
	}

	print $retval;
	return $exit_code;
}

// Generate an HTML page with the 4 (day,week,month,year) graphs from the graph's data
function create_page($info_graph)
{
	global $schedule;
	global $colors;

	$txt = html_start($info_graph['Title']);

	foreach($schedule['Label'] as $period => $label)
	{
		$txt .= "<br>
			<table border=0 CELLPADDING=0>
				<th BGCOLOR=$colors[Title]><center><b>Graphique $label</b></center></th>
				<tr BGCOLOR=$colors[Cell]><td>
					<table border=0 CELLPADDING=10>
						<tr><td>
							<a href=".$_SERVER['PHP_SELF']."?action=zoom&target=$_GET[target]&begin="
									.strtotime($schedule['Timestamps'][$period])."&end=".time().">
								<img src=".create_graph($info_graph['Graph'],$period)." border=0>
							</a>
						</td></tr>
					</table>
				</td></tr>
				<tr BGCOLOR=$colors[Footer]><td align=right>
					<a href='#' onclick='window.scrollTo(0,0); return false'><small>Haut</small></a>
				</td></tr>
			</table>
		";
	}

	$txt .= html_end($info_graph['Caption']);

	print $txt;
}

// Generate an HTML page with one graph between the specified timesamps.
function create_zoom($info_graph, $begin, $end)
{
	global $colors;

	$txt = html_start($info_graph['Title']);

	$txt .= "
		<script language='javascript'>
			// Mega global vars used by dynzoom.js (Yep, this is really ugly !)
			var ts_begin = $begin;
			var ts_end = $end;

			function slide(way)
			{
				var win_length=ts_end-ts_begin;
				if(way == 'next')
				{
					ts_begin += win_length;
					ts_end += win_length;
				}
				else if(way == 'previous')
				{
					ts_begin -= win_length;
					ts_end -= win_length;
				}
				open('$_SERVER[PHP_SELF]?action=zoom&target=$_GET[target]&begin='+ts_begin+'&end='+ts_end, '_self');
			} 
		</script><script language='javascript' src='dynzoom.js'></script>
	";

	$txt .= "<br>
		<table border=0 CELLPADDING=0>
			<th BGCOLOR=$colors[Title]><center><b>Graphique du ".date("D j M Y H:i:s",$begin)." au ".date("D j M Y H:i:s",$end)."</b></center></th>
			<tr BGCOLOR=$colors[Cell]><td>
				<table border=0 CELLPADDING=10>
					<tr><td id=zoomGraphImage>
						<div id='zoomBox' style='position:absolute; overflow:none; left:0px; top:0px; width:0px; height:0px; 
							visibility:visible; background:green; filter:alpha(opacity=50); -moz-opacity:0.5; -khtml-opacity:0.5; opacity:0.5'></div>
				         <div id='zoomSensitiveZone' style='position:absolute; overflow:none; left:0px; top:0px; width:0px; height:0px; 
							visibility:visible; cursor:crosshair; background:blue; filter:alpha(opacity=0); -moz-opacity:0; -khtml-opacity:0; opacity:0' 
							oncontextmenu='return false'></div>
						<img id=graphImage src=".create_graph($info_graph['Graph'],"zoom",false,$begin,$end)." border=0>
					</td></tr>
				</table>
			</td></tr>
			<tr BGCOLOR=$colors[Slider]><td align=right>
				<a href=\"javascript:slide('previous')\"><img src=btn_previous.png border=0></a>
				<a href=\"javascript:slide('next')\"><img src=btn_next.png border=0></a><br>
			</td></tr>
			<tr BGCOLOR=$colors[Footer]><td align=center>
				<form method='get' action='$_SERVER[PHP_SELF]' style='display: inline'>
					Du: <input type=text name=begin value='".date("Y/m/d H:i:s",$begin)."'>
					Au: <input type=text name=end value='".date("Y/m/d H:i:s",$end)."'>
					<input type=hidden name=action value=zoom>
					<input type=hidden name=target value=$_GET[target]>
					<input type=submit value='Ok'>
				</form>
			</td></tr>
		</table><br>
		
	";

	$txt .= html_end($info_graph['Caption']);

	print $txt;
}

// Generate an HTML page with the first (day) graph for each graphs' datas
function create_index($graphs,$title,$caption)
{
	global $colors;
	
	// Flag to put graph on a new row
	$new_row=false;

	$txt = html_start($title)."<table border=0 CELLPADDING=0><tr><td>";

	foreach($graphs as $name => $info_graph)
	{

		$txt .= "<br>
            <table border=0 CELLPADDING=0>
                <th BGCOLOR=$colors[Title]><center><b>$info_graph[Label]</b></center></th>
                <tr BGCOLOR=$colors[Cell]><td>
                    <table border=0 CELLPADDING=0>
                        <tr><td>
							<a href=".$_SERVER['PHP_SELF']."?action=details&target=$name>
								<img src=".create_graph($info_graph['Graph'],"day",true)." border=0>
							</a>
						</td></tr>
                    </table>
                </td></tr>
				<tr><td align=right>
					<a href='#' onclick='window.scrollTo(0,0); return false'><small>Haut</small></a>
				</td></tr>
            </table>
        ";

		$txt .= $new_row ? "</td></tr><tr><td>" : "</td><td>";
		$new_row=!$new_row;  // Toggle flag for next graph
	}

	$txt .= "</td></tr></table>".html_end($caption);

	print $txt;
}

//Delete images of specified graph
function delete_graph($graph)
{
	global $schedule,$debug;

	foreach($schedule['Label'] as $period => $name)
	{
		$filename = $graph['Prefix']."-".$period.".png";
		if(file_exists($filename))
		{
			if($debug) print "Deleting $filename";
			unlink($filename) or die("Cannot delete $filename.");
		}
	}
}

// HTML Subs
//---------------------------------------------------------------------------------

function html_start($title)
{
	global $colors;
	
	$txt = "<html><head>
				<title>$title</title>
				<META NAME='Author' LANG='fr' CONTENT='Nicolas AGIUS'>
				<META NAME='Copyright' CONTENT='GNU General Public Licence'>
				<META NAME='Generator' CONTENT='MyRRD v1.2'>
			</head><body bgcolor=$colors[Background]>";

	$txt .= "<table width=100%>
				<tr><td>
					<h1>$title</h1>
				</td><td align=right valign=top>
					<small>
						[ <a href=".$_SERVER['PHP_SELF'].">Index</a> ] 
						[ <a href=".$_SERVER['PHP_SELF']."?refresh=true&".$_SERVER['QUERY_STRING'].">Raffraichir</a> ]
					</small>
				</td></tr>
			</table>";
	$txt .= "<br>Les statistiques de <b>";
	$txt .= $_SERVER['SERVER_NAME'];
	$txt .= "</b> ont &eacute;t&eacute; mises &agrave; jour le <b>";
	$txt .= date("l d F Y \&\a\g\\r\a\v\e\; H:i");
	$txt .= "</b><br><br>";
	
	return $txt;
}

function html_end($caption)
{
	$txt  = "<br>";
	if(strlen($caption)>0) $txt .= "<blockquote><p>$caption</p></blockquote>";
	$txt .= "<center><small><i>
				MyRRD v1.2 &nbsp; - &nbsp; Copyleft 2007-2011 GNU/GPL  &nbsp; - &nbsp; Nicolas Agius
			 </i></small></center>";
	$txt .= "</body></html>";
	
	return $txt;
}

?>
